///////////////////////////////////////
/// 640x480 version!
/// test VGA with hardware video input copy to VGA
// compile with
// gcc tenMatrix.c -o tenMatrix -lm
///////////////////////////////////////
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ipc.h> 
#include <sys/shm.h> 
#include <sys/mman.h>
#include <sys/time.h> 
#include <math.h> 
#include "address_map_arm_brl4.h"

// video display
#define SDRAM_BASE            0xC0000000
#define SDRAM_END             0xC3FFFFFF
#define SDRAM_SPAN			  0x04000000
// characters
#define FPGA_CHAR_BASE        0xC9000000 
#define FPGA_CHAR_END         0xC9001FFF
#define FPGA_CHAR_SPAN        0x00002000
/* Cyclone V FPGA devices */
#define HW_REGS_BASE          0xff200000
//#define HW_REGS_SPAN        0x00200000 
#define HW_REGS_SPAN          0x00005000 

#define rgbConv(R,G,B) B+(G<<5)+(R<<11)

// 16-bit primary colors
#define red  (0+(0<<5)+(31<<11))
#define dark_red (0+(0<<5)+(15<<11))
#define green (0+(63<<5)+(0<<11))
#define dark_green (0+(31<<5)+(0<<11))
#define blue (31+(0<<5)+(0<<11))
#define dark_blue (15+(0<<5)+(0<<11))
#define yellow (0+(63<<5)+(31<<11))
#define cyan (31+(63<<5)+(0<<11))
#define magenta (31+(0<<5)+(31<<11))
#define black (0x0000)
#define gray (15+(31<<5)+(51<<11))
#define white (0xffff)

typedef signed int fix ;
#define multfix(a,b) ((fix)((((signed long long)(a))*((signed long long)(b)))>>15))
#define float2fix(a) ((fix)((a)*32768.0)) // 2^15
#define fix2float(a) ((float)(a)/32768.0)
#define absfix(a) abs(a)
#define int2fix(a) ((fix)(a << 15))
#define fix2int(a) ((int)(a >> 15))
#define divfix(a,b) ((fix)(((( signed long long)(a) << 15 / (b)))))
#define squarefix(a) multfix(a,a)

// pixel macro
#define VGA_PIXEL(x,y,color) do{\
	int  *pixel_ptr ;\
	pixel_ptr = (int*)((char *)vga_pixel_ptr + (((y)*640+(x))<<1)) ; \
	*(short *)pixel_ptr = (color);\
} while(0)

typedef struct {
    float x;
    float y;
	float z;
	float w;
} Vector;

enum signalStates { //this ENUM contains the two values that can be in the zeroth position of the shared SRAM *(sram_ptr)
    HPSWORKING = 0,  //0 means HPS is drawing or writing initial values
    FPGAWORKING	= 1	//1 means fpga is populating
};

void VGA_text (int, int, char *);
void VGA_text_clear();
void VGA_box (int, int, int, int, short);
void VGA_rect (int, int, int, int, short);
void VGA_line(int, int, int, int, short) ;
void VGA_Vline(int, int, int, short) ;
void VGA_Hline(int, int, int, short) ;
void VGA_disc (int, int, int, short);
void VGA_circle (int, int, int, int);

void printSRAMPoints(volatile int * sram_ptr, float * FPGAXOLD, float * FPGAYOLD, float * FPGAZOLD);
float getRandomFloat(int lowerBound, int upperBound);
void rotate_x(float theta, float C[3][3]);
void rotate_y(float theta, float C[3][3]);
void rotate_z(float theta, float C[3][3]);
Vector getRandomVector();
void rotateVector(Vector * vector, float C[3][3]);
void norm(Vector * vector);
float mag(Vector * vector);
int compare(const void *a, const void *b);

#define DEG2RAD(deg) deg * M_PI/180
#define NUMOFMULT 10
#define numOfRows 3000//35000 //number of pixels per module * numof modules
#define numToDisplay 3000
#define VGAWIDTH 640
#define VGAHEIGHT 480

// uS per frame
#define FRAME_RATE 16666 //1/30 * 1000000 or 1/60 if 60fps

// the light weight buss base
void *h2p_lw_virtual_base;

// pixel buffer
volatile unsigned int * vga_pixel_ptr = NULL ;
void *vga_pixel_virtual_base;

// character buffer
volatile unsigned int * vga_char_ptr = NULL ;
void *vga_char_virtual_base;

// RAM FPGA command buffer
volatile int * sram_ptr = NULL ;
volatile int * sram_ptr2 = NULL ;
volatile int * sram_ptr3 = NULL ;
volatile int * sram_ptr4 = NULL ;
volatile int * sram_ptr5 = NULL ;
volatile int * sram_ptr6 = NULL ;
volatile int * sram_ptr7 = NULL ;
volatile int * sram_ptr8 = NULL ;
volatile int * sram_ptr9 = NULL ;
volatile int * sram_ptr10 = NULL ;

void *sram_virtual_base;

// /dev/mem file id
int fd;

// measure time
struct timeval hpsStart, hpsEnd, FPGAStart, FPGAEnd, t1, t2;
double elapsedTime, elapsedTime2;
double spareTime;
	
int main(void) {  
	// === get FPGA addresses ==================
    // Open /dev/mem
	if( ( fd = open( "/dev/mem", ( O_RDWR | O_SYNC ) ) ) == -1 ) 	{
		printf( "ERROR: could not open \"/dev/mem\"...\n" );
		return( 1 );
	}
    
    // get virtual addr that maps to physical
	h2p_lw_virtual_base = mmap( NULL, HW_REGS_SPAN, ( PROT_READ | PROT_WRITE ), MAP_SHARED, fd, HW_REGS_BASE );	
	if( h2p_lw_virtual_base == MAP_FAILED ) {
		printf( "ERROR: mmap1() failed...\n" );
		close( fd );
		return(1);
	}
    
	// === get VGA char addr =====================
	// get virtual addr that maps to physical
	vga_char_virtual_base = mmap( NULL, FPGA_CHAR_SPAN, ( 	PROT_READ | PROT_WRITE ), MAP_SHARED, fd, FPGA_CHAR_BASE );	
	if( vga_char_virtual_base == MAP_FAILED ) {
		printf( "ERROR: mmap2() failed...\n" );
		close( fd );
		return(1);
	}
    
    // Get the address that maps to the FPGA LED control 
	vga_char_ptr =(unsigned int *)(vga_char_virtual_base);

	// === get VGA pixel addr ====================
	// get virtual addr that maps to physical
	vga_pixel_virtual_base = mmap( NULL, SDRAM_SPAN, ( PROT_READ | PROT_WRITE ), MAP_SHARED, fd, SDRAM_BASE);	
	if( vga_pixel_virtual_base == MAP_FAILED ) {
		printf( "ERROR: mmap3() failed...\n" );
		close( fd );
		return(1);
	}
    
    // Get the address that maps to the FPGA pixel buffer
	vga_pixel_ptr =(unsigned int *)(vga_pixel_virtual_base);

	// ===========================================

    // === get RAM FPGA parameter addr =========
	sram_virtual_base = mmap( NULL, FPGA_ONCHIP_SPAN, ( PROT_READ | PROT_WRITE ), MAP_SHARED, fd, FPGA_ONCHIP_BASE); //fp	
	
	if( sram_virtual_base == MAP_FAILED ) {
		printf( "ERROR: mmap3() failed...\n" );
		close( fd );
		return(1);
	}
    // Get the address that maps to the RAM buffer
	sram_ptr =(int *)(sram_virtual_base);
	sram_ptr2 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE2);
	sram_ptr3 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE3);
	sram_ptr4 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE4);
	sram_ptr5 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE5);
	sram_ptr6 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE6);
	sram_ptr7 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE7);
	sram_ptr8 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE8);
	sram_ptr9 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE9);
	sram_ptr10 =(int *)(sram_virtual_base + FPGA_ONCHIP_BASE10);
	
	char time_string[50] ;
	
	float rotMatrix[3][3] = {0};
    rotate_y(DEG2RAD(1), rotMatrix);

    Vector vectors[numOfRows] = {0};

    int i;
	unsigned short temp, color;

    for(i = 0; i < numOfRows; ++i) {
        vectors[i] = getRandomVector();
    }
	
	//Indicate to FPGA that HPS is doing work
	unsigned char j;
	for(j = 0; j < NUMOFMULT; ++j) {
		*((int *)(sram_virtual_base + j*SRAM_OFFSET)) = HPSWORKING;
	}

	//Fill sram with values from vectors
	for(j = 0; j < NUMOFMULT; ++j) {
		for(i = 0; i < numToDisplay/NUMOFMULT; ++i) {
			*((int *)(sram_virtual_base + j*SRAM_OFFSET) + 3*i + 1)= float2fix(vectors[i+j*(numToDisplay/NUMOFMULT)].x);
			*((int *)(sram_virtual_base + j*SRAM_OFFSET) + 3*i + 2)= float2fix(vectors[i+j*(numToDisplay/NUMOFMULT)].y);
			*((int *)(sram_virtual_base + j*SRAM_OFFSET) + 3*i + 3)= float2fix(vectors[i+j*(numToDisplay/NUMOFMULT)].z);
		}
	}
    
	// clear the screen
	VGA_box (0, 0, VGAWIDTH-1, VGAHEIGHT-1, 0x0000);
	VGA_text_clear();

	sprintf(time_string, "Number of Matrix Multiplications = %d ", numToDisplay);
	VGA_text (23, 1, time_string);

	float FPGAXOLD[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD[numToDisplay/NUMOFMULT] = {0};

	float FPGAXOLD2[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD2[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD2[numToDisplay/NUMOFMULT] = {0};
    
	float FPGAXOLD3[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD3[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD3[numToDisplay/NUMOFMULT] = {0};

	float FPGAXOLD4[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD4[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD4[numToDisplay/NUMOFMULT] = {0};

	float FPGAXOLD5[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD5[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD5[numToDisplay/NUMOFMULT] = {0};

	float FPGAXOLD6[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD6[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD6[numToDisplay/NUMOFMULT] = {0};

	float FPGAXOLD7[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD7[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD7[numToDisplay/NUMOFMULT] = {0};
    
	float FPGAXOLD8[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD8[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD8[numToDisplay/NUMOFMULT] = {0};

	float FPGAXOLD9[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD9[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD9[numToDisplay/NUMOFMULT] = {0};

	float FPGAXOLD10[numToDisplay/NUMOFMULT] = {0};
	float FPGAYOLD10[numToDisplay/NUMOFMULT] = {0};
	float FPGAZOLD10[numToDisplay/NUMOFMULT] = {0};

	//Indicate to FPGA that it is free to work
	for(j = 0; j < NUMOFMULT; ++j) {
		*((int *)(sram_virtual_base + j*SRAM_OFFSET)) = FPGAWORKING; //this index indicates to fpga that we are writing //1 means fpga is populating//0 means HPS is drawing or writing initial values
	}

	gettimeofday(&FPGAStart, NULL);

	while(1) {
		if (*(sram_ptr) == HPSWORKING) { //FPGA has signaled it has done a calculation
			// Stop FPGA Timer
			gettimeofday(&FPGAEnd, NULL);
			elapsedTime = FPGAEnd.tv_usec - FPGAStart.tv_usec;
			sprintf(time_string, "FPGA Time (us) = %4.0f   ", elapsedTime);
			VGA_text (12, 10, time_string);
			sprintf(time_string, "Split onto %2d modules ", NUMOFMULT);
			VGA_text (12, 12, time_string);
			
			gettimeofday(&t1, NULL);

            //printSRAMStuff
            printSRAMPoints(sram_ptr, FPGAXOLD, FPGAYOLD, FPGAZOLD);
            printSRAMPoints(sram_ptr2, FPGAXOLD2, FPGAYOLD2, FPGAZOLD2);
            printSRAMPoints(sram_ptr3, FPGAXOLD3, FPGAYOLD3, FPGAZOLD3);
            printSRAMPoints(sram_ptr4, FPGAXOLD4, FPGAYOLD4, FPGAZOLD4);
            printSRAMPoints(sram_ptr5, FPGAXOLD5, FPGAYOLD5, FPGAZOLD5);
			printSRAMPoints(sram_ptr6, FPGAXOLD6, FPGAYOLD6, FPGAZOLD6);
            printSRAMPoints(sram_ptr7, FPGAXOLD7, FPGAYOLD7, FPGAZOLD7);
            printSRAMPoints(sram_ptr8, FPGAXOLD8, FPGAYOLD8, FPGAZOLD8);
            printSRAMPoints(sram_ptr9, FPGAXOLD9, FPGAYOLD9, FPGAZOLD9);
            printSRAMPoints(sram_ptr10, FPGAXOLD10, FPGAYOLD10, FPGAZOLD10);
            //

			// start HPS timer
			gettimeofday(&hpsStart, NULL);
			for(i = 0; i < numOfRows; ++i) {
				VGA_box((int)(vectors[i].x + (2*VGAWIDTH)/3) , (int)(vectors[i].y + VGAHEIGHT/2), (int)(vectors[i].x +  (2*VGAWIDTH)/3) + 2 , (int)(vectors[i].y + VGAHEIGHT/2) + 2, 0);
				rotateVector(&vectors[i], rotMatrix);
				norm(&vectors[i]); //since we are rotating about y
					// Rotated Values may not be in og Z range, scale them to be
				if(vectors[i].z > 60) {
					temp = 31;
				} else if (vectors[i].z < -60) {
					temp = 0;
				} else {
					temp = ((vectors[i].z + 60.0)/120.0) * 31.0;
				}
				color = rgbConv(temp, temp, temp);
				VGA_box((int)(vectors[i].x +  (2*VGAWIDTH)/3) , (int)(vectors[i].y + VGAHEIGHT/2), (int)(vectors[i].x +  (2*VGAWIDTH)/3) + 2 , (int)(vectors[i].y + VGAHEIGHT/2) + 2, color);
        	}

			// Stop HPS timer
			gettimeofday(&hpsEnd, NULL);
			elapsedTime = hpsEnd.tv_usec - hpsStart.tv_usec;
			sprintf(time_string, "HPS Time (uS) = %4.0f   ", elapsedTime);
			VGA_text (50, 10, time_string);
			// set frame rate

			gettimeofday(&t2, NULL);
			// delay in accordance with frame rate
			spareTime = FRAME_RATE - (t2.tv_usec - t1.tv_usec);
			if (spareTime > FRAME_RATE) {
				spareTime = 0;
			}
			usleep(spareTime);

			
			for(j = 0; j < NUMOFMULT; ++j) {
				*((int *)(sram_virtual_base + j*SRAM_OFFSET)) = FPGAWORKING; //this index indicates to fpga that we are writing //1 means fpga is populating//0 means HPS is drawing or writing initial values
			}
			// //We are ready to start timing the FPGA
			//Start FPGA Timer
			gettimeofday(&FPGAStart, NULL);
		}
		
	} // end while(1)
}

void printSRAMPoints(volatile int * sram_ptr_func, float * FPGAXOLD, float * FPGAYOLD, float * FPGAZOLD) {
    // Get Values from FPGA SRAM0 and draw them
    int i;
    float FPGAX, FPGAY, FPGAZ;
	unsigned short temp, color;
    for(i = 0; i < numToDisplay/NUMOFMULT; ++i) {
        //clear old
        VGA_box((int)(FPGAXOLD[i] + VGAWIDTH/3) , (int)(FPGAYOLD[i] + VGAHEIGHT/2), (int)(FPGAXOLD[i] + VGAWIDTH/3) + 2 , (int)(FPGAYOLD[i] + VGAHEIGHT/2) + 2, 0);
        //get new
        FPGAX = fix2float( *(sram_ptr_func + 3*i + 1) );
        FPGAXOLD[i] = FPGAX;
        FPGAY = fix2float( *(sram_ptr_func + 3*i + 2) );
        FPGAYOLD[i] = FPGAY;
        FPGAZ = fix2float( *(sram_ptr_func + 3*i + 3) );
        FPGAZOLD[i] = FPGAZ;
        
        // Rotated Values may not be in og Z range, scale them to be
        if(FPGAZ > 60) {
            temp = 31;
        } else if (FPGAZ < -60) {
            temp = 0;
        } else {
            temp = ((FPGAZ + 60.0)/120.0) * 31.0;
        }
        color = rgbConv(temp, temp, temp);
        VGA_box((int)(FPGAX + VGAWIDTH/3) , (int)(FPGAY + VGAHEIGHT/2), (int)(FPGAX + VGAWIDTH/3) + 2 , (int)(FPGAY + VGAHEIGHT/2) + 2, color);
    }

}

/****************************************************************************************
 * Subroutine to read a pixel from the video input 
****************************************************************************************/
// int  video_in_read_pixel(int x, int y){
	// char  *pixel_ptr ;
	// pixel_ptr = (char *)video_in_ptr + ((y)<<9) + (x) ;
	// return *pixel_ptr ;
// }

/****************************************************************************************
 * Subroutine to read a pixel from the VGA monitor 
****************************************************************************************/
int  VGA_read_pixel(int x, int y){
	char  *pixel_ptr ;
	pixel_ptr = (char *)vga_pixel_ptr + ((y)*640) + (x) ;
	return *pixel_ptr ;
}

/****************************************************************************************
 * Subroutine to send a string of text to the VGA monitor 
****************************************************************************************/
void VGA_text(int x, int y, char * text_ptr)
{
  	volatile char * character_buffer = (char *) vga_char_ptr ;	// VGA character buffer
	int offset;
	/* assume that the text string fits on one line */
	offset = (y << 7) + x;
	while ( *(text_ptr) )
	{
		// write to the character buffer
		*(character_buffer + offset) = *(text_ptr);	
		++text_ptr;
		++offset;
	}
}

/****************************************************************************************
 * Subroutine to clear text to the VGA monitor 
****************************************************************************************/
void VGA_text_clear()
{
  	volatile char * character_buffer = (char *) vga_char_ptr ;	// VGA character buffer
	int offset, x, y;
	for (x=0; x<79; x++){
		for (y=0; y<59; y++){
	/* assume that the text string fits on one line */
			offset = (y << 7) + x;
			// write to the character buffer
			*(character_buffer + offset) = ' ';		
		}
	}
}

/****************************************************************************************
 * Draw a filled rectangle on the VGA monitor 
****************************************************************************************/
#define SWAP(X,Y) do{int temp=X; X=Y; Y=temp;}while(0) 

void VGA_box(int x1, int y1, int x2, int y2, short pixel_color)
{
	char  *pixel_ptr ; 
	int row, col;

	/* check and fix box coordinates to be valid */
	if (x1>639) x1 = 639;
	if (y1>479) y1 = 479;
	if (x2>639) x2 = 639;
	if (y2>479) y2 = 479;
	if (x1<0) x1 = 0;
	if (y1<0) y1 = 0;
	if (x2<0) x2 = 0;
	if (y2<0) y2 = 0;
	if (x1>x2) SWAP(x1,x2);
	if (y1>y2) SWAP(y1,y2);
	for (row = y1; row <= y2; row++)
		for (col = x1; col <= x2; ++col)
		{
			//640x480
			VGA_PIXEL(col, row, pixel_color);
			//pixel_ptr = (char *)vga_pixel_ptr + (row<<10)    + col ;
			// set pixel color
			//*(char *)pixel_ptr = pixel_color;		
		}
}

/****************************************************************************************
 * Draw a filled circle on the VGA monitor 
****************************************************************************************/
void VGA_disc(int x, int y, int r, short pixel_color)
{
	char  *pixel_ptr ; 
	int row, col, rsqr, xc, yc;
	
	rsqr = r*r;
	
	for (yc = -r; yc <= r; yc++)
		for (xc = -r; xc <= r; xc++)
		{
			col = xc;
			row = yc;
			// add the r to make the edge smoother
			if(col*col+row*row <= rsqr+r){
				col += x; // add the center point
				row += y; // add the center point
				//check for valid 640x480
				if (col>639) col = 639;
				if (row>479) row = 479;
				if (col<0) col = 0;
				if (row<0) row = 0;
				VGA_PIXEL(col, row, pixel_color);
				//pixel_ptr = (char *)vga_pixel_ptr + (row<<10) + col ;
				// set pixel color
				//nanosleep(&delay_time, NULL);
				//draw_delay();
				//*(char *)pixel_ptr = pixel_color;
			}
					
		}
}

// =============================================
// === Draw a line
// =============================================
//plot a line 
//at x1,y1 to x2,y2 with color 
//Code is from David Rodgers,
//"Procedural Elements of Computer Graphics",1985
void VGA_line(int x1, int y1, int x2, int y2, short c) {
	int e;
	signed int dx,dy,j, temp;
	signed int s1,s2, xchange;
     signed int x,y;
	char *pixel_ptr ;
	
	/* check and fix line coordinates to be valid */
	if (x1>639) x1 = 639;
	if (y1>479) y1 = 479;
	if (x2>639) x2 = 639;
	if (y2>479) y2 = 479;
	if (x1<0) x1 = 0;
	if (y1<0) y1 = 0;
	if (x2<0) x2 = 0;
	if (y2<0) y2 = 0;
        
	x = x1;
	y = y1;
	
	//take absolute value
	if (x2 < x1) {
		dx = x1 - x2;
		s1 = -1;
	}

	else if (x2 == x1) {
		dx = 0;
		s1 = 0;
	}

	else {
		dx = x2 - x1;
		s1 = 1;
	}

	if (y2 < y1) {
		dy = y1 - y2;
		s2 = -1;
	}

	else if (y2 == y1) {
		dy = 0;
		s2 = 0;
	}

	else {
		dy = y2 - y1;
		s2 = 1;
	}

	xchange = 0;   

	if (dy>dx) {
		temp = dx;
		dx = dy;
		dy = temp;
		xchange = 1;
	} 

	e = ((int)dy<<1) - dx;  
	 
	for (j=0; j<=dx; j++) {
		//video_pt(x,y,c); //640x480
		VGA_PIXEL(x, y, c);
		//pixel_ptr = (char *)vga_pixel_ptr + (y<<10)+ x; 
		// set pixel color
		//*(char *)pixel_ptr = c;	
		 
		if (e>=0) {
			if (xchange==1) x = x + s1;
			else y = y + s2;
			e = e - ((int)dx<<1);
		}

		if (xchange==1) y = y + s2;
		else x = x + s1;

		e = e + ((int)dy<<1);
	}
}

// FUNCTION DESCRIPTION:
//   Realize a rotation about the x-axis by an angle of theta
//
// INPUTS:
//   theta = rotation angle (rad)
//
// OUTPUTS:
//   C   = 3X3 Rotation matrix (dimless)
//
// Example:  C = rotate_x(30 * pi/180, C) - Rotate by 30° about the y-axis
//
void rotate_x(float theta, float C[3][3]) {
    C[0][0] = 1;  C[0][1] = 0;           C[0][2] = 0;
    C[1][0] = 0;  C[1][1] = cos(theta);  C[1][2] = -sin(theta);;
    C[2][0] = 0;  C[2][1] = sin(theta);  C[2][2] = cos(theta);
}

// FUNCTION DESCRIPTION:
//   Realize a rotation about the y-axis by an angle of theta
//
// INPUTS:
//   theta = rotation angle (rad)
//
// OUTPUTS:
//   C   = 3X3 Rotation matrix (dimless)
//
// Example:  C = rotate_y(30 * pi/180, C) - Rotate by 30° about the y-axis
//
void rotate_y(float theta, float C[3][3]) {
    C[0][0] = cos(theta);  C[0][1] = 0;     C[0][2] = sin(theta);
    C[1][0] = 0;           C[1][1] = 1;     C[1][2] = 0;
    C[2][0] = -sin(theta); C[2][1] = 0;     C[2][2] = cos(theta);
}

// FUNCTION DESCRIPTION:
//   Realize a rotation about the z-axis by an angle of theta
//
// INPUTS:
//   theta = rotation angle (rad)
//
// OUTPUTS:
//   C   = 3X3 Rotation matrix (dimless)
//
// Example:  C = rotate_z(30 * pi/180, C) - Rotate by 30° about the y-axis
//
void rotate_z(float theta, float C[3][3]) {
    C[0][0] = cos(theta);  C[0][1] = -sin(theta);  C[0][2] = 0;
    C[1][0] = sin(theta);  C[1][1] = cos(theta);   C[1][2] = 0;
    C[2][0] = 0;           C[2][1] = 0;            C[2][2] = 1;
}

/*
 * Function: getRandomDouble
 * Purpose:  gets a random double
 * Argument:  lowebound and the upperbound to create a range that the random number can be in
 * Returns:  a random double number between the lower and upper bound
 */
float getRandomFloat(int lowerBound, int upperBound) {
    float rangeOfReals = upperBound - lowerBound;
    float normalizedRand = rand() / (RAND_MAX + 1.0);
    return normalizedRand * rangeOfReals + lowerBound;
}

/*
 * Function: getRandomPoint 
 * Purpose:  puts random numbers into x y and z members, calculates magnitude of vector
 * Returns:  a vector struct
 */
Vector getRandomVector() {
    Vector vector;
    // vector.x = getRandomFloat(-320, 320);
    // vector.y = getRandomFloat(-240, 240);
    // vector.z = getRandomFloat(-200, 200);
	vector.x = getRandomFloat(-60, 60);
    vector.y = getRandomFloat(-60, 60);
    vector.z = getRandomFloat(-60, 60);
	vector.w = mag(&vector);
    return vector;
}

void rotateVector(Vector * vector, float C[3][3]) {
    // multiply point by rotation matrix
	float x = vector->x;
	float y = vector->y;
	float z = vector->z;
    vector->x = x * C[0][0] + y * C[0][1] + z * C[0][2];
    vector->y = x * C[1][0] + y * C[1][1] + z * C[1][2];
    vector->z = x * C[2][0] + y * C[2][1] + z * C[2][2];
}

/*
 * Function: norm 
 * Purpose:  Normalizes a vector using it original length and current length.
 * Returns:  nothing
 */
void norm(Vector * vector) {
	float newW = mag(vector);
	vector->x = vector->x/newW * vector->w; //make it original length
	vector->y = vector->y/newW * vector->w;
	vector->z = vector->z/newW * vector->w;
}

/*
 * Function: mag 
 * Purpose:  gets the magnitude of a vector
 * Returns:  the magnitude of a vector as a float
 */
float mag(Vector * vector) {
	float w = sqrt(vector->x * vector->x + vector->z * vector->z); //unitize current vector
	return w;
}

/*
 * Function: compare 
 * Purpose:  compares the z value of a vector struct
 * Returns:  the magnitude of a vector as a float
 */
int compare(const void *a, const void *b) {
    Vector *vectorA = (Vector *)a;
    Vector *vectorB = (Vector *)b;
	if (vectorA->z < vectorB->z)
      return -1;
   else if (vectorA->z > vectorB->z)
      return 1;
   else
      return 0;
}